  
"
    __________________________________________________
    @Alejandro F. Reimondo [mailTo:aleReimondo@smalltalking.net] 22/02/00
    This proyect sources are property of : Alejandro F. Reimondo [mailTo:aleReimondo@smalltalking.net]
    Its use is NOT legal without written rights.
    Amenabar 3476 Piso: 2 Dto: B.
    Ciudad de Buenos Aires.
    Argentina.
    Tel: (54)-1-703-5586.
    __________________________________________________
"!

Object subclass: #TaskBarIcon
  instanceVariableNames: 
    ' notifyIconData window '
  classVariableNames: ''
  poolDictionaries: ''  !

ObjectWindow subclass: #TaskBarWindow
  instanceVariableNames: 
    ' icons '
  classVariableNames: 
    ' WmNotifyIcon '
  poolDictionaries: ''  !

TaskBarWindow class instanceVariableNames: ' current ' !


!TaskBarIcon class methods !
 
constructEventsTriggered
	"Private - Answer the set of events that instances of the receiver can trigger."

	^super constructEventsTriggered
		add: #mouseMoved ;
		add: #button1Down ;
		add: #button1Up ;
		add: #clicked ;
		add: #button2Down ;
		add: #button2Up ;
		add: #doubleClicked ;
		add: #unknownEvent: ;
		yourself!
 
default
	" Returns an instance of the receiver set with default values. "

	^self new default!
   
initializeStructures
	" Private - Initializes the structures for the lowlevel interfase.
	TaskBar initializeStructures
	"

	Dynalink32
        registerType: 'NotifyIconData'
        as:#(
            (cbSize dword)
            (hWnd ulong)
            (uID ulong)
            (uFlags ulong)
            (uCallbackMessage ulong)
            (hIcon ulong)
            (szTip 64 asCstring)
        );
        yourself.!

notifyIconData
	" Private - Returns an instance of the lowlevel structure. "

	| result |
	result := Dynalink32 structureNamed: 'NotifyIconData'.
	result isNil ifTrue: [
		self initializeStructures.
		^self notifyIconData
	].
	^result! !



!TaskBarIcon methods !

add
	" Add the receiver to the system taskbar. "

	^ShellDLL current
		shellNotifyIcon: self nimAdd asParameter
		with: self notifyIconData asParameter!
   
addFlag: aFlag
	"Add aFlag to the receiver's flags."

	self flags: self flags | aFlag!
   
button1Down
	"The mouse has been pressed over the icon."

    self triggerEvent: #button1Down.!
  
button1Up
	"The mouse has been released over the icon."

    self triggerEvent: #button1Up.
    self triggerEvent: #clicked.!
   
button2Down
	"The mouse has been pressed over the icon."

    self triggerEvent: #button2Down.!
  
button2Up
	"The mouse has been released over the icon."

    self triggerEvent: #button2Up.!
 
default
	" Set the receiver with default values. "

	self
		id: 77; "we use this number for default taskbar icon id"
        icon: ViewManager icon;
        tip: Smalltalk product
			,' ',Smalltalk versionNumberString
			,' [',Smalltalk imageName,']';
		useDefaultWindow;
		yourself
		!
   
delete
	" Remove the receiver from the system taskbar. "

	^ShellDLL current
		shellNotifyIcon: self nimDelete asParameter
		with: self notifyIconData asParameter!

doubleClicked
	"The mouse has been doubleClicked."

    self triggerEvent: #doubleClicked.!
  
flags
	"Returns the receiver's flags."

	^self notifyIconData uFlags!

flags: anInteger
	"Sets the receiver's flags."

	^self notifyIconData uFlags: anInteger!
 
icon: anIcon
	" Sets the receiver's icon. "

	self notifyIconData hIcon: anIcon asParameter.
	anIcon isNil
	ifTrue: [ self removeFlag: self nifIcon ]
	ifFalse: [ self addFlag: self nifIcon ].!
  
id
	"Returns the application identifier of the receiver."

	^self notifyIconData uID!

id: anInteger
	"Set the application identifier of the receiver."

	self notifyIconData uID: anInteger!
   
initializeNotifyIconData
	" Private - Initializes the receiver's notifyIconData"

	notifyIconData := self class notifyIconData.
	notifyIconData
		cbSize: notifyIconData sizeInBytes;
		yourself!
 
modify
	" Modify the receiver in the system taskbar. "

	^ShellDLL current
		shellNotifyIcon: self nimModify asParameter
		with: self notifyIconData asParameter!
  
mouseMoved
	"The mouse has been moved over the window."

    self triggerEvent: #mouseMoved.!

nifIcon
	" Private - Returns the NIF_ICON flag. "

	^2!
  
nifMessage
	" Private - Returns the NIF_MESSAGE flag. "

	^1!

nifTip
	" Private - Returns the NIF_TIP flag. "

	^4!

nimAdd
	" Private - Returns the NIF_ADD message. "

	^0!
 
nimDelete
	" Private - Returns the NIF_DELETE message. "

	^2!
   
nimModify
	" Private - Returns the NIF_MODIFY message. "

	^1!
   
notifyIconData
    "Returns the instance var of the receiver
     for external manipulation"

	notifyIconData isNil ifTrue: [ self initializeNotifyIconData ].
    ^notifyIconData!

notifyIconData: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    notifyIconData := anObject!
   
remove
	" Remove the receiver from the system taskbar. "

	^self delete!
 
removeFlag: aFlag
	"Remove aFlag from the receiver's flags."

	self flags: self flags & aFlag bitInvert!
 
tip: aString
	"Set the ToolTip text of the receiver."

	aString isNil ifTrue: [ ^self tip: String new ].
	self notifyIconData szTip: aString asAsciiZ.
	aString isEmpty
	ifTrue: [ self removeFlag: self nifTip ]
	ifFalse: [ self addFlag: self nifTip ].!
  
unknownEvent: longInteger
	"An unKnown event has been occurred on the receiver."

    self triggerEvent: #unknownEvent: with: longInteger.!
  
update
	" Modify the receiver in the system taskbar. "

	^self modify!
   
useDefaultWindow
	" Use the default window for the receiver. "

	self window: TaskBarWindow current.
		!

window
    "Returns the instance var of the receiver
     for external manipulation"
    ^window!

window: aWindow
    " Sets the receiver's window. "

    aWindow isNil
    ifTrue: [
        self notifyIconData
            hWnd: 0;
            uCallbackMessage: 0.
        self removeFlag: self nifMessage.
		window notNil ifTrue: [
			window unRegister: self.
		]
    ]
    ifFalse: [
        self notifyIconData
            hWnd: aWindow handle;
            uCallbackMessage: aWindow wmNotifyIcon.
        self addFlag: self nifMessage.
		aWindow register: self.
    ].

	window := aWindow.! !



!TaskBarWindow class methods !

current
    "Returns the instance var of the receiver
     for external manipulation"

	current isNil ifTrue: [ self startUp ].
    ^current!
  
current: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    current := anObject!
 
currentOrNil
    "Returns the instance var of the receiver
     for external manipulation"
    ^current!
 
exit
	"Private - Shut down the receiver.
	Close current window if needed."

    current notNil ifTrue: [ current close ].!
  
startUp
	" StartUp the receiver. "

    | aWindow |
    aWindow := self new open.
    WmNotifyIcon notNil ifTrue: [
		OperatingSystemEventsExtra removeKey: WmNotifyIcon ifAbsent: [ ].
	    WmNotifyIcon := nil.
	].
    self wmNotifyIcon.
    OperatingSystemEventsExtra at: WmNotifyIcon put: #wmNotifyIcon:with: .
    self current: aWindow!
   
wmNotifyIcon
	" Private - Answer the message number for the WmNotifyIcon message. "

    | id |
    WmNotifyIcon isNil
	ifTrue: [
        id := UserLibrary
			registerWindowMessage: 'WM_NOTIFYICON' asParameter.
		id = 0 ifTrue: [ ^self osError ].
		WmNotifyIcon := id
	].
    ^WmNotifyIcon! !



!TaskBarWindow methods !
 
close
	"Private - Close the window."

	self deleteAllIcons.
	self == self class currentOrNil ifTrue: [
		self class current: nil
	].
	^super close!
  
delete: aTaskBarIcon
	"Delete aTaskBarIcon."

	aTaskBarIcon delete.
	self unRegister: aTaskBarIcon.!

deleteAll: aCollection
	"Delete all the icons in aCollection."

	aCollection asSet do: [:each| self delete: each ].!
 
deleteAllIcons
	"Delete all the icons managed by the receiver."

	self deleteAll: self registeredIcons!
  
icons
    "Returns the instance var of the receiver
     for external manipulation"

	icons isNil ifTrue: [ self initializeIcons ].
    ^icons!

initializeIcons
	" Private - Initializes the receiver's icons"

	icons := Dictionary new.!
   
open
	"Private - Open the receiver window."

    self label: Smalltalk product,' - TaskBarWindow.'.
    ^super open!

register: aTaskBarIcon
	"Register aTaskBarIcon in the receiver."

	^self icons at: aTaskBarIcon id put: aTaskBarIcon!

registeredIcons
	"Returns the icons registered in the receiver."

	^self icons values!
   
unRegister: aTaskBarIcon
	"UnRegister aTaskBarIcon in the receiver."

	^self icons removeKey: aTaskBarIcon id ifAbsent: []!
  
wmNotifyIcon
	" Returns the WM callback notification message. "

	^WmNotifyIcon!
 
wmNotifyIcon: wordInteger with: longInteger
	"Private - Process the WmNotifyIcon callback message."

	| selected |
	selected := self icons
		at: wordInteger ifAbsent: [ ^self ]. "unknown icon."
	512 == longInteger ifTrue: [ ^selected mouseMoved ].
	513 == longInteger ifTrue: [ ^selected button1Down ].
	514 == longInteger ifTrue: [ ^selected button1Up ].
	515 == longInteger ifTrue: [ ^selected doubleClicked ].
	516 == longInteger ifTrue: [ ^selected button2Down ].
	517 == longInteger ifTrue: [ ^selected button2Up ].
	^selected unknownEvent: longInteger! !


! ShellDLL methods !
shellNotifyIcon: dwMessage with: pnid
    <api: Shell_NotifyIconA ulong struct boolean>
    ^self invalidArgument! !